@@ -27,8 +27,7 @@ from scipy.signal import convolve2d
 from numpy import log
 
 
-
-def get_coordinates(image_size, window_size, overlap):
+def get_coordinates(image_size, search_area_size, window_size, overlap):
     """Compute the x, y coordinates of the centers of the interrogation windows.
 
     Parameters
@@ -38,6 +37,9 @@ def get_coordinates(image_size, window_size, overlap):
         first element is number of rows, second element is
         the number of columns.
 
+    search_area_size: int
+        the size of the search area windows.
+
     window_size: int
         the size of the interrogation windows.
 
@@ -59,20 +61,25 @@ def get_coordinates(image_size, window_size, overlap):
     """
 
     # get shape of the resulting flow field
-    field_shape = get_field_shape(image_size, window_size, overlap)
+    field_shape = get_field_shape(image_size, search_area_size, window_size, overlap)
 
-    # compute grid coordinates of the interrogation window centers
-    # compute grid coordinates of the interrogation window centers
-    x = np.arange( field_shape[1] )*(window_size-overlap) + window_size/2.0
-    y = np.arange( field_shape[0] )*(window_size-overlap) + window_size/2.0
+    # compute grid coordinates of the search area window centers
+    # compute grid coordinates of the search area window centers
+    x = np.arange(field_shape[1]) * (window_size - overlap) + (search_area_size - 1)/2.0
+    y = np.arange(field_shape[0]) * (window_size - overlap) + (search_area_size - 1)/2.0
+
+    # moving coordinates further to the center, so that the points at the extreme left/right or top/bottom
+    # have the same distance to the window edges. For simplicity only integer movements are allowed.
+    x += (image_size[1] - 1 - ((field_shape[1] - 1) * (window_size - overlap) + (search_area_size - 1))) // 2
+    y += (image_size[0] - 1 - ((field_shape[0] - 1) * (window_size - overlap) + (search_area_size - 1))) // 2
 
     return np.meshgrid(x, y)
 
 
-def get_field_shape(image_size, window_size, overlap):
+def get_field_shape(image_size, search_area_size, window_size, overlap):
     """Compute the shape of the resulting flow field.
 
-    Given the image size, the interrogation window size and
+    Given the image size, the interrogation window size, the search_area size and
     the overlap size, it is possible to calculate the number
     of rows and columns of the resulting flow field.
 
@@ -83,6 +90,9 @@ def get_field_shape(image_size, window_size, overlap):
         first element is number of rows, second element is
         the number of columns.
 
+    search_area_size: int
+        the size of the search area window.
+
     window_size: int
         the size of the interrogation window.
 
@@ -97,8 +107,8 @@ def get_field_shape(image_size, window_size, overlap):
         the shape of the resulting flow field
     """
 
-    return ((image_size[0] - window_size) // (window_size - overlap) + 1,
-            (image_size[1] - window_size) // (window_size - overlap) + 1)
+    return ((image_size[0] - search_area_size) // (window_size - overlap) + 1,
+            (image_size[1] - search_area_size) // (window_size - overlap) + 1)
 
 
 def moving_window_array(array, window_size, overlap):
@@ -236,12 +246,10 @@ def find_subpixel_peak_position(corr, subpixel_method='gaussian'):
 
     # initialization
     # default_peak_position = (np.floor(corr.shape[0] / 2.), np.floor(corr.shape[1] / 2.))
-    default_peak_position = (0,0)
-
+    default_peak_position = (0, 0)
     
     # the peak locations
     peak1_i, peak1_j, dummy = find_first_peak(corr)
-    
 
     try:
         # the peak and its neighbours: left, right, down, up
@@ -386,18 +394,19 @@ def correlate_windows(window_a, window_b, corr_method='fft', nfftx=0, nffty=0):
         window_b = np.conj(window_b[::-1, ::-1])
         if nfftx == 0:
             nfftx = nextpower2(window_b.shape[0] + window_a.shape[0])  
-        if nffty  == 0:
+        if nffty == 0:
             nffty = nextpower2(window_b.shape[1] + window_a.shape[1]) 
         
         f2a = rfft2(normalize_intensity(window_a), s=(nfftx, nffty))
         f2b = rfft2(normalize_intensity(window_b), s=(nfftx, nffty))
         corr = irfft2(f2a * f2b).real
-        corr = corr[:window_a.shape[0] + window_b.shape[0], 
-                    :window_b.shape[1] + window_a.shape[1]]
+        corr = corr[:window_a.shape[0] + window_b.shape[0] - 1,
+                    :window_b.shape[1] + window_a.shape[1] - 1]
         return corr
     elif corr_method == 'direct':
         return convolve2d(normalize_intensity(window_a),
-        normalize_intensity(window_b[::-1, ::-1]), 'full')
+                          normalize_intensity(window_b[::-1, ::-1]), 'full')
+
     else:
         raise ValueError('method is not implemented')
 
@@ -510,7 +519,6 @@ def extended_search_area_piv(
     """
     
     # check the inputs for validity
-    
     if search_area_size == 0:
         search_area_size = window_size
     
@@ -519,77 +527,84 @@ def extended_search_area_piv(
     
     if search_area_size < window_size:
         raise ValueError('Search size cannot be smaller than the window_size')
-    
         
     if (window_size > frame_a.shape[0]) or (window_size > frame_a.shape[1]):
         raise ValueError('window size cannot be larger than the image')
         
     # get field shape
-    n_rows, n_cols = get_field_shape((frame_a.shape[0], frame_a.shape[1]), 
-                                     search_area_size, overlap)
+    n_rows, n_cols = get_field_shape(frame_a.shape,
+                                     search_area_size, window_size, overlap)
 
+    # create empty arrays for deformation in x and y direction
     u, v = np.zeros((n_rows, n_cols)), np.zeros((n_rows, n_cols))
     
     # if we want sig2noise information, allocate memory
     if sig2noise_method is not None:
         sig2noise = np.zeros((n_rows, n_cols))
-    
-    # loop over the interrogation windows
-    # i, j are the row, column indices of the center of each interrogation
-    # window
-    for k in range(n_rows):
-        # range(range(search_area_size/2, frame_a.shape[0] - search_area_size/2, window_size - overlap ):
-        for m in range(n_cols):
-            # range(search_area_size/2, frame_a.shape[1] - search_area_size/2 , window_size - overlap ):
-            
-            
-            # Select first the largest window, work like usual from the top left corner
-            # the left edge goes as: 
-            # e.g. 0, (search_area_size - overlap), 2*(search_area_size - overlap),....
-            
-            il = k*(search_area_size - overlap)
-            ir = il + search_area_size
-            
-            # same for top-bottom
-            jt = m*(search_area_size - overlap)
-            jb = jt + search_area_size
-            
-            # pick up the window in the second image
-            window_b = frame_b[il:ir, jt:jb]            
-            
-            # now shift the left corner of the smaller window inside the larger one
-            il += (search_area_size - window_size)//2
-            # and it's right side is just a window_size apart
-            ir = il + window_size
-            # same same
-            jt += (search_area_size - window_size)//2
-            jb =  jt + window_size
 
-            window_a = frame_a[il:ir, jt:jb]
+    # shift for x and y coordinates of the search area windows so that the centers of search area windows have
+    # the same distances to the image edge at all sides. For simplicity only shifts by integers are allowed
+    x_centering = (frame_a.shape[1] - 1 - ((n_cols - 1) * (window_size - overlap) + (search_area_size - 1))) // 2
+    y_centering = (frame_a.shape[0] - 1 - ((n_rows - 1) * (window_size - overlap) + (search_area_size - 1))) // 2
+
+    # iterate through interrogation widows and search areas
+    for k in range(n_rows):
+        for m in range(n_cols):
+
+            # centers of search area. (window_size - overlap) defines the distance between each center
+            # and (search_area_size - 1)/2.0 moves the center points away from the left or top image edge
+            y = k * (window_size - overlap) + (search_area_size - 1)/2.0
+            x = m * (window_size - overlap) + (search_area_size - 1)/2.0
+
+            # moving the coordinates a bit to the center, to guarantee that the distance
+            #  of a extreme point at the image edges is symmetric all all edges
+            x += x_centering
+            y += y_centering
+
+            # left, right, top, bottom indices of the search area edges
+            # note that x - (search_area_size +/- 1)/2  always returns an integer due to the definition of x and y
+            # see also "get_coordinates()"
+            il = int(y - (search_area_size - 1) / 2.0)
+            ir = int(y + (search_area_size + 1) / 2.0)
+            it = int(x - (search_area_size - 1) / 2.0)
+            ib = int(x + (search_area_size + 1) / 2.0)
+            # picking the search area from frame b
+            window_b = frame_b[il:ir, it:ib]
+
+            # left, right, top, bottom indices of the interrogation window
+            # Sometimes the interrogation window cannot be placed in the middle of the search area, e.g.
+            # in the case of window_size=3 search_area_size=4. In this case the interrogation window
+            # is shifted 0.5 pixels to the left/top, which is achieved by rounding
+            # the indices down during the int() conversion
+            il = int(y - (window_size - 1) / 2)
+            ir = int(y + (window_size + 1) / 2)
+            it = int(x - (window_size - 1) / 2)
+            ib = int(x + (window_size + 1) / 2)
+            # picking the interrogation window from frame a
+            window_a = frame_a[il:ir, it:ib]
 
             if np.any(window_a):
                 corr = correlate_windows(window_a, window_b,
-                                         corr_method=corr_method, 
+                                         corr_method=corr_method,
                                          nfftx=nfftx, nffty=nffty)
-#                 plt.figure()
-#                 plt.contourf(corr)
-#                 plt.show()
+                #                 plt.figure()
+                #                 plt.contourf(corr)
+                #                 plt.show()
                 # get subpixel approximation for peak position row and column index
-                row, col = find_subpixel_peak_position(corr, 
-                                                        subpixel_method=subpixel_method)
-                                
-                row -= (search_area_size + window_size - 1)//2
-                col -= (search_area_size + window_size - 1)//2
-    
+                row, col = find_subpixel_peak_position(corr,
+                                                       subpixel_method=subpixel_method)
+
+                row -= (search_area_size + window_size - 1) // 2
+                col -= (search_area_size + window_size - 1) // 2
+
                 # get displacements, apply coordinate system definition
-                u[k,m],v[k,m] = -col, row 
-                
+                u[k, m], v[k, m] = -col, row
+
                 # get signal to noise ratio
                 if sig2noise_method is not None:
-                    sig2noise[k,m] = sig2noise_ratio(
+                    sig2noise[k, m] = sig2noise_ratio(
                         corr, sig2noise_method=sig2noise_method, width=width)
-                
-    
+
     # return output depending if user wanted sig2noise information
     if sig2noise_method is not None:
         return u/dt, v/dt, sig2noise
@@ -600,5 +615,6 @@ def extended_search_area_piv(
 def nextpower2(i):
     """ Find 2^n that is equal to or greater than. """
     n = 1
-    while n < i: n *= 2
-    return n
\ No newline at end of file
+    while n < i:
+        n *= 2
+    return n
