@@ -1,6 +1,3 @@
-#!/usr/bin/python
-
-
 """A module for spurious vector detection."""
 
 __licence__ = """
@@ -62,6 +59,8 @@ def global_val( u, v, u_thresholds, v_thresholds ):
         a boolean array. True elements corresponds to outliers.
         
     """
+
+    np.warnings.filterwarnings('ignore')
     
     ind = np.logical_or(\
           np.logical_or(u < u_thresholds[0], u > u_thresholds[1]), \
@@ -123,7 +122,7 @@ def global_std( u, v, std_threshold = 3 ):
     
     return u, v, mask
 
-def sig2noise_val( u, v, sig2noise, threshold = 1.3):
+def sig2noise_val( u, v, sig2noise, w=None, threshold = 1.3):
     """Eliminate spurious vectors from cross-correlation signal to noise ratio.
     
     Replace spurious vectors with zero if signal to noise ratio
@@ -131,27 +130,32 @@ def sig2noise_val( u, v, sig2noise, threshold = 1.3):
     
     Parameters
     ----------
-    u : 2d np.ndarray
-        a two dimensional array containing the u velocity component.
+    u : 2d or 3d np.ndarray
+        a two or three dimensional array containing the u velocity component.
         
-    v : 2d np.ndarray
-        a two dimensional array containing the v velocity component.
+    v : 2d or 3d np.ndarray
+        a two or three dimensional array containing the v velocity component.
         
     sig2noise : 2d np.ndarray
-        a two dimensional array containing the value  of the signal to 
+        a two or three dimensional array containing the value  of the signal to
         noise ratio from cross-correlation function.
-        
+    w : 2d or 3d np.ndarray
+        a two or three dimensional array containing the w (in z-direction) velocity component.
     threshold: float
         the signal to noise ratio threshold value.
         
     Returns
     -------
-    u : 2d np.ndarray
-        a two dimensional array containing the u velocity component, 
+    u : 2d or 3d np.ndarray
+        a two or three dimensional array containing the u velocity component,
         where spurious vectors have been replaced by NaN.
         
-    v : 2d np.ndarray
-        a two dimensional array containing the v velocity component, 
+    v : 2d or 3d  np.ndarray
+        a two or three dimensional array containing the v velocity component,
+        where spurious vectors have been replaced by NaN.
+
+    w : 2d or 3d  np.ndarray
+        optional, a two or three dimensional array containing the w (in z-direction) velocity component.
         where spurious vectors have been replaced by NaN.
         
     mask : boolean 2d np.ndarray 
@@ -162,16 +166,18 @@ def sig2noise_val( u, v, sig2noise, threshold = 1.3):
     R. D. Keane and R. J. Adrian, Measurement Science & Technology,1990, 1, 1202-1215.
     
     """
-    
+
     ind = sig2noise < threshold
 
     u[ind] = np.nan
     v[ind] = np.nan
-    
-    mask = np.zeros(u.shape, dtype=bool)
-    mask[ind] = True
-    
-    return u, v, mask
+    if isinstance(w, np.ndarray):
+        w[ind] = np.nan
+        return u, v, w, ind
+
+    return u, v, ind
+
+
 
 def local_median_val( u, v, u_threshold, v_threshold, size=1 ):
     """Eliminate spurious vectors with a local median threshold.
